import { CodeSyntaxElement, SyntaxDefinition, languages } from 'code-syntax';
import { SuperImposeElement } from 'super-impose';
import { TextAreaCodeElement } from 'textarea-code';
export { languages };
declare global {
    interface ShadowRootInit {
        customElements?: CustomElementRegistry;
    }
}
/**
 * CodeEdit HTML/JSX Interface
 */
export interface HTMLCodeEditElement {
    /** The editor's value */
    value?: string;
    /** Language for syntax highlighting */
    language?: string;
    /** Passing a syntax definition regexp manually */
    syntax?: SyntaxDefinition | Promise<{
        default: SyntaxDefinition;
    }>;
    /** Theme to use */
    theme?: string;
    /** Tab size */
    tabsize?: number;
    /** Tab style */
    tabstyle?: 'tabs' | 'spaces';
    /** Comments are defined as a tuple-like string: '// /* *\/' first item is single comment and second third are multiline comments */
    comments?: string;
    /** Autofocus to the element when page loads */
    autofocus?: boolean;
    /** Whether to autoresize to the height of the contents */
    autoresize?: boolean;
}
/**
 * CodeEdit custom element
 *
 * ```js
 * import { CodeEditElement } from 'code-edit'
 * customElements.define('code-edit', CodeEditElement)
 * ```
 *
 * ```html
 * <code-edit autofocus autoresize language="js" theme="deep"> ... </code-edit>
 * ```
 */
export declare class CodeEditElement extends HTMLElement {
    static get observedAttributes(): string[];
    nodes: Node[];
    pins: {
        pre: HTMLPreElement;
        parent: SuperImposeElement;
        textarea: TextAreaCodeElement;
        syntax: CodeSyntaxElement;
        syntaxTextNode: Text;
    };
    update: () => void;
    constructor();
    get scrollLeft(): number;
    get scrollTop(): number;
    get value(): string;
    set value(v: string);
    set syntax(def: SyntaxDefinition | Promise<{
        default: SyntaxDefinition;
    }>);
    focus(): void;
    updateValue(): void;
    autoResize(): void;
    autoFocus(): void;
    attributeChangedCallback(name: string, _oldValue: string | null, newValue: string | null): void;
}
export default CodeEditElement;
